#!/usr/bin/env node
/**
 * User: garcia.wul (garcia.wul@alibaba-inc.com)
 * Date: 2014/10/14
 * Time: 15:42
 *
 */

var ArgumentParser = require('argparse').ArgumentParser;
var chalk = require("chalk");
var fs = require("fs");
var path = require("path");
var _ = require("underscore");
var Index = require("../index");

var argumentParser = new ArgumentParser({
    version: require("../package.json").version,
    addHelp: true,
    description: "command line tool for cmd-nice"
});

argumentParser.addArgument(["--configFile"], {
    help: "config file",
    type: "string",
    required: false,
    dest: "configFile"
});

argumentParser.addArgument(["--input"], {
    help: "input files",
    type: "string",
    required: true,
    dest: "inputFiles",
    nargs: "*"
});

var args = argumentParser.parseArgs();

var alias = {};
var aliasPaths = {};
if (fs.existsSync(args.configFile)) {
    var configContent = fs.readFileSync(args.configFile, "utf-8");
    configContent = eval(configContent);
    alias = configContent.alias;
    aliasPaths = configContent.paths;
}

_.each(args.inputFiles, function(fileName) {
    var dependencyUtils = new Index.DependencyUtils({
        alias: alias,
        aliasPaths: aliasPaths,
        rootPath: process.cwd()
    });
    var dependencies = dependencyUtils.analyseDependencies(
        fs.readFileSync(fileName, "utf-8"),
        fs.realpathSync(fileName)
    );
    console.log(chalk.cyan("\n" + fileName + "'s dependencies:"));
    _.each(dependencies, function(name) {
        console.log(name);
    });
});